/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
***/

"use strict";
window.OW_GoogleMapLocation = function ($)
{
    
    var location = function (fieldName, addressFieldId, mapElementId)
    {
        this.map;
        this.marker;
        this.mapElement = $("#" + mapElementId);
        this.bounds;

        this.fieldId = addressFieldId;
        this.mapElementId = mapElementId;
        this.zoom = 9;

        this.countryRestriction = '';
        this.removeValue;
        this.region;
        this.customMarkerIcon;
        this.fieldName = fieldName;
        
        this.latitudeField;
        this.longitudeField;
        this.northEastLat;
        this.northEastLng;
        this.southWestLat;
        this.southWestLng;
        this.addressField;
        this.jsonField;
        this.removeValue;
        this.dispalyMap = mapElementId;
    }

    location.prototype = {
        initialize: function(params) {

            var self = this;
            var lat = params['lat'];
            var lng = params['lng'];
            var northEastLat = params['northEastLat'];
            var northEastLng = params['northEastLng'];
            var southWestLat = params['southWestLat'];
            var southWestLng = params['southWestLng'];

            if ( params['customMarkerIcon'] )
            {
                this.customMarkerIcon = params['customMarkerIcon'];
            }

            this.region = params['region'];

            if ( this.dispalyMap ) {
                this.map = new window.GOOGLELOCATION.Map(this.mapElementId);
                this.map.initialize();
                this.map.disableDefaultUI(true);
                this.map.disableInput(true);
                this.map.disablePanning(true);
                this.map.disableZooming(true);

                if (lat || lng) {
                    this.mapElement.show();
                    this.map.setCenter(lat, lng);

                    if ((northEastLat || northEastLng) && (southWestLat || southWestLng)) {
                        this.map.fitBounds([southWestLat, southWestLng, northEastLat, northEastLng]);
                    }

                    this.map.addPoint(lat, lng, null, null, null, this.customMarkerIcon);
                    this.map.createMarkerCluster();
                    this.map.resize();
                    this.map.resetLastBounds();
                }
            }

            this.countryRestriction = params['countryRestriction'];

            $(function () {
                $("#" + self.fieldId + "_delete_icon").click(function () {
                    self.deleteValue.bind(self)();
                });
                
                self.latitudeField = $('input[name="' + self.fieldName + '[latitude]"]');
                self.longitudeField = $('input[name="' + self.fieldName + '[longitude]"]');
                self.northEastLat = $('input[name="' + self.fieldName + '[northEastLat]"]');
                self.northEastLng = $('input[name="' + self.fieldName + '[northEastLng]"]');
                self.southWestLat = $('input[name="' + self.fieldName + '[southWestLat]"]');
                self.southWestLng = $('input[name="' + self.fieldName + '[southWestLng]"]');
                self.addressField = $('input[name="' + self.fieldName + '[address]"]');
                self.jsonField = $('input[name="' + self.fieldName + '[json]"]');
                self.removeValue = $('input[name="' + self.fieldName + '[remove]"]');

                OW_GoogleMapLocationAutocomplete(self.fieldId, GOOGLELOCATION.AutocompleteParams(self));
            });
        },
        
        isValid: function() {
            if( this.jsonField.val() || this.removeValue.val() )
            {
                return true;
            }
            
            return false;
        },
        
        deleteValue: function(item)
        {
            this.hideDeleteIcon();

            $('#' + this.fieldId).val('');
            this.addressField.val('');
            this.longitudeField.val('');
            this.latitudeField.val('');
            this.northEastLat.val('');
            this.northEastLng.val('');
            this.southWestLat.val('');
            this.southWestLng.val('');
            this.removeValue.val(true);

            this.jsonField.val('');
        },
        
        hideDeleteIcon: function (item)
        {
            if ( this.dispalyMap )
            {
                $("#" + this.mapElementId).hide();
            }
            
            $("#" + this.fieldId + "_icon").show();
            $("#" + this.fieldId + "_delete_icon").hide();
        },

        showDeleteIcon: function (item)
        {
            if ( this.dispalyMap )
            {
                $("#" + this.mapElementId).show();
            }
            
            $("#" + this.fieldId + "_icon").hide();
            $("#" + this.fieldId + "_delete_icon").css("display", "inline");
        }

    };

    return location;
}(locationJquey)