<?php
/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
***/
class MEMBERSHIP_CMP_SetMembership extends OW_Component
{
    /**
     * Class constructor
     */
    public function __construct( $userId )
    {
        parent::__construct();

        if ( !OW::getUser()->isAuthorized('membership') )
        {
            $this->setVisible(false);

            return;
        }

        $user = BOL_UserService::getInstance()->findUserById($userId);

        if ( !$user )
        {
            $this->setVisible(false);

            return;
        }

        $accTypeName = $user->getAccountType();
        $accType = BOL_QuestionService::getInstance()->findAccountTypeByName($accTypeName);

        $service = MEMBERSHIP_BOL_MembershipService::getInstance();
        $authService = BOL_AuthorizationService::getInstance();

        $types = $service->getTypeList($accType->id);

        /* @var $defaultRole BOL_AuthorizationRole */
        $defaultRole = $authService->getDefaultRole();
        $default = array('value' => 'default', 'label' => $authService->getRoleLabel($defaultRole->name));
        $this->assign('default', $default);

        $memberships = array();
        foreach ( $types as &$ms )
        {
            $memberships[$ms->id] = $service->getMembershipTitle($ms->roleId);
        }
        $this->assign('memberships', $memberships);

        $current = $service->getUserMembership($userId);
        $this->assign('current', $current);

        if ( $current )
        {
            $this->assign('remaining', $service->getRemainingPeriod($current->expirationStamp));
        }

        $form = new MEMBERSHIP_CLASS_SetMembershipForm();
        $this->addForm($form);

        $form->getElement('userId')->setValue($userId);
        $form->bindJsFunction(
            Form::BIND_SUCCESS,
            "function(data){
                if ( data.result ) {
                    OW.info(data.msg);
                    document.setMembershipFloatBox.close();
                }
                else {
                    OW.error(data.msg);
                }
             }"
        );

        $script =
        '$("input[name=type]").change(function(){
            if ( $(this).val() == "default" ) {
                $("#period-cont").css("display", "none");
            }
            else {
                $("#period-cont").css("display", "table-row");
            }
        });';

        OW::getDocument()->addOnloadScript($script);
    }
}