<?php
/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
***/
class MEMBERSHIP_CTRL_Ajax extends OW_ActionController
{
    public function __construct()
    {
        parent::__construct();

        if ( !OW::getRequest()->isPost() || !OW::getRequest()->isAjax() )
        {
            throw new Redirect403Exception();
        }
    }

    public function set()
    {
        if ( !OW::getUser()->isAuthorized('membership') )
        {
            throw new Redirect403Exception();
        }
        
        $form = new MEMBERSHIP_CLASS_SetMembershipForm();

        if ( !$form->isValid($_POST) )
        {
            throw new Redirect403Exception();
        }

        $service = MEMBERSHIP_BOL_MembershipService::getInstance();
        $values = $form->getValues();

        if ( !$values['userId'] || !$values['type'] )
        {
            throw new Redirect403Exception();
        }

        $userId = (int) $values['userId'];
        $typeId = $values['type'];

        $user = BOL_UserService::getInstance()->findUserById($userId);
        if ( !$user )
        {
            throw new Redirect403Exception();
        }

        $result = array(
            'result' => true,
            'msg' => OW::getLanguage()->text('membership', 'user_membership_updated')
        );

        if ( $typeId == "default" )
        {
            $service->setDefaultMembership($userId);

            exit(json_encode($result));
        }

        $membership = $service->findTypeById($typeId);

        if ( !$membership )
        {
            throw new Redirect403Exception();
        }

        if ( empty($values['period']) )
        {
            $result = array(
                'result' => false,
                'msg' => OW::getLanguage()->text('membership', 'period_required')
            );

            exit(json_encode($result));
        }

        $ms = new MEMBERSHIP_BOL_MembershipUser();
        $ms->userId = $userId;
        $ms->typeId = $typeId;
        $ms->recurring = 0;
        $ms->expirationStamp = time() + (int) $values['period'] * 24 * 3600;

        $service->setUserMembership($ms);

        exit(json_encode($result));
    }

    public function deleteType()
    {
        if ( !OW::getUser()->isAdmin() )
        {
            throw new Redirect403Exception();
        }

        if ( empty($_POST['typeId']) )
        {
            throw new Redirect403Exception();
        }

        $service = MEMBERSHIP_BOL_MembershipService::getInstance();
        $authService = BOL_AuthorizationService::getInstance();

        $typeId = (int) $_POST['typeId'];
        $deleteType = isset($_POST['type']) && in_array($_POST['type'], array('custom', 'default')) ? $_POST['type'] : 'default';
        $newTypeId = isset($_POST['newTypeId']) && (int) $_POST['newTypeId'] ? (int) $_POST['newTypeId'] : null;

        $type = $service->findTypeById($typeId);
        if ( !$type )
        {
            throw new Redirect403Exception();
        }

        if ( in_array($deleteType, array('default', 'custom')) )
        {
            $page = 1; $limit = 50;
            while ( true )
            {
                $users = $service->getUserObjectListByMembershipType($typeId, $page, $limit);

                if ( !$users )
                {
                    break;
                }

                foreach ( $users as $userMembership )
                {
                    if ( $deleteType == 'custom' && $newTypeId )
                    {
                        $userMembership->id = null;
                        $userMembership->typeId = $newTypeId;
                        $service->setUserMembership($userMembership);
                    }
                    else if ( $deleteType == 'default' )
                    {
                        $service->deleleUserMembership($userMembership);
                        $authService->deleteUserRole($userMembership->userId, $type->roleId);
                        $authService->assignDefaultRoleToUser($userMembership->userId);
                    }
                }

                $page++;
            }
        }

        $service->deleteTypeWithPlans($typeId);
        OW::getFeedback()->info(OW::getLanguage()->text('membership', 'types_deleted'));

        exit(json_encode(array('result' => true)));
    }

    public function deletePlans()
    {
        if ( !OW::getUser()->isAdmin() )
        {
            throw new Redirect403Exception();
        }

        if ( empty($_POST['plans']) )
        {
            throw new Redirect403Exception();
        }

        $plans = $_POST['plans'];

        $service = MEMBERSHIP_BOL_MembershipService::getInstance();
        foreach ( $plans as $planId )
        {
            $service->deletePlan($planId);
        }

        exit(json_encode(array('result' => true)));
    }
}