<?php
/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
***/
class PROTECTEDPHOTOS_BOL_PasswordDao extends OW_BaseDao
{
    const ALBUM_ID = 'albumId';

    private static $instance;

    public static function getInstance()
    {
        if ( null === self::$instance )
        {
            self::$instance = new self();
        }

        return self::$instance;
    }

    public function getTableName()
    {
        return OW_DB_PREFIX . 'protectedphotos_passwords';
    }

    public function getDtoClassName()
    {
        return 'PROTECTEDPHOTOS_BOL_Password';
    }

    public function findByAlbumId( $albumId )
    {
        if ( empty($albumId) )
        {
            return null;
        }

        $example = new OW_Example();
        $example->andFieldEqual(self::ALBUM_ID, $albumId);

        return $this->findObjectByExample($example);
    }

    public function isAlbumProtected( $albumId )
    {
        if ( empty($albumId) )
        {
            return false;
        }

        $example = new OW_Example();
        $example->andFieldEqual(self::ALBUM_ID, $albumId);

        return (int) $this->countByExample($example) > 0;
    }

    /**
     * @param array $ids
     * @return PROTECTEDPHOTOS_BOL_Password[]
     */
    public function findByAlbumIds( array $ids )
    {
        if ( count($ids) === 0 )
        {
            return array();
        }

        $example = new OW_Example();
        $example->andFieldInArray(self::ALBUM_ID, $ids);

        return $this->findListByExample($example);
    }

    public function deleteByAlbumId( $albumId )
    {
        if ( empty($albumId) )
        {
            return null;
        }

        $example = new OW_Example();
        $example->andFieldEqual(self::ALBUM_ID, $albumId);

        return $this->deleteByExample($example);
    }

    /**
     * @param $userId
     * @param $privacies
     * @return PROTECTEDPHOTOS_BOL_Password[]
     */
    public function findUserPasswordByPrivacy( $userId, array $privacies )
    {
        if ( empty($userId) || count($privacies) === 0 )
        {
            return array();
        }

        $sql = 'SELECT `p`.*
            FROM `' . $this->getTableName() . '` AS `p`
                INNER JOIN `' . OW_DB_PREFIX . 'photo_album` AS `a` ON(`p`.`albumId` = `a`.`id`)
            WHERE `p`.`privacy` IN (' . $this->dbo->mergeInClause($privacies) . ') AND `a`.`userId` = :userId';

        return $this->dbo->queryForObjectList($sql, $this->getDtoClassName(), array(
            'userId' => $userId
        ));
    }
}