<?php

/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
***/
class WINKS_CLASS_ConsoleEventHandler
{
    CONST CONSOLE_ITEM_KEY = 'wink_requests';

    private static $classInstance;

    public static function getInstance()
    {
        if ( !isset(self::$classInstance) )
        {
            self::$classInstance = new self();
        }

        return self::$classInstance;
    }

    private $service;

    private function __construct()
    {
        $this->service = WINKS_BOL_Service::getInstance();
    }

    public function init()
    {
        $eventManager = OW::getEventManager();

        $eventManager->bind('console.collect_items', array($this, 'collectItems'));
        $eventManager->bind('console.ping', array($this, 'ping'));
        $eventManager->bind('console.load_list', array($this, 'loadList'));
    }

    public function collectItems( BASE_CLASS_ConsoleItemCollector $event )
    {
        if ( OW::getUser()->isAuthenticated() )
        {
            $item = new WINKS_CMP_ConsoleWinkRequests();
            $userId = OW::getUser()->getId();
            $activeModes = $this->service->getActiveModes();

            if ( $this->service->countWinksForUser($userId, array(WINKS_BOL_WinksDao::STATUS_ACCEPT, WINKS_BOL_WinksDao::STATUS_WAIT), null, $activeModes) === 0 && $this->service->countWinkBackedByUserId($userId, $activeModes) === 0 )
            {
                $item->setIsHidden(true);
            }

            $event->addItem($item, 5);
        }
    }

    public function ping( BASE_CLASS_ConsoleDataEvent $event )
    {
        $userId = OW::getUser()->getId();
        $activeModes = $this->service->getActiveModes();

        $event->setItemData(self::CONSOLE_ITEM_KEY, array('counter' => array(
            'all' => $this->service->countWinksForUser($userId, array(WINKS_BOL_WinksDao::STATUS_WAIT), null, $activeModes),
            'new' => $this->service->countWinksForUser($userId, array(WINKS_BOL_WinksDao::STATUS_WAIT), 0, $activeModes)
        )));
    }

    public function loadList( BASE_CLASS_ConsoleListEvent $event )
    {
        $params = $event->getParams();
        $userId = OW::getUser()->getId();

        if ( $params['target'] != self::CONSOLE_ITEM_KEY )
        {
            return;
        }

        $activeModes = json_decode(OW::getConfig()->getValue('mailbox', 'active_modes'));

        $winks = $this->service->findWinkList($userId, $params['offset'], 10, $activeModes);
        $viewedIds = array();
        $language = OW::getLanguage();

        $language->addKeyForJs('winks', 'msg_accept_request');
        $language->addKeyForJs('winks', 'msg_ignore_request');
        $mode = is_array($activeModes) ? (in_array('chat', $activeModes)) ? 'chat' : 'mail' : 'chat';

        foreach ( $winks as $wink )
        {
            $viewedIds[] = $wink->getId();

            $avatar = BOL_AvatarService::getInstance()->getDataForUserAvatars(array($wink->getUserId()), true, true, true, false);

            $item = new BASE_CMP_ConsoleListIpcItem();
            $item->setAvatar($avatar[$wink->getUserId()]);
            $item->setKey('wink-item-' . $wink->getId());

            $userUrl = OW::getRouter()->urlForRoute('base_user_profile', array('username'=>BOL_UserService::getInstance()->getUserName($wink->getUserId())));
            $displayName = BOL_UserService::getInstance()->getDisplayName($wink->getUserId());
            //OW::getRouter()->addRoute(new OW_Route('mailbox_conversation', 'messages/mail/:convId', 'MAILBOX_CTRL_Messages', 'index'));
            if ( $wink->getUserId() == $userId )
            {
                if ( $wink->getStatus() == WINKS_BOL_WinksDao::STATUS_ACCEPT && $wink->getWinkback() )
                {
                    $string = $language->text('winks', 'console_wink_accept_item', array(
                        'userUrl' => OW::getRouter()->urlForRoute('base_user_profile', array('username'=>BOL_UserService::getInstance()->getUserName($wink->getPartnerId()))),
                        'displayName' => BOL_UserService::getInstance()->getDisplayName($wink->getPartnerId()))
                    );

                  if ( $mode == 'mail' || $wink->messageType == 'mail' )
                  {
                        $item->setToolbar(array(
                            array(
                                'label' => $language->text('winks', 'send_message'),
                                'url' => OW::getRouter()->urlForRoute('mailbox_conversation', array('convId'=>$wink->getConversationId()))
                            )
                        ));
                  }
                  else
                  {
                      $item->setToolbar(array(
                            array(
                                'label' => $language->text('winks', 'send_message'),
                                'onclick' => 'OW.trigger(\'mailbox.open_dialog\',{convId:' . $wink->getConversationId() . ',opponentId:' . $wink->getPartnerId() . ',mode:\'' . $mode . '\'});return false;'
                            )
                        ));
                  }
                }
                else
                {
                    continue;
                }
            }
            elseif ( $wink->getStatus() == WINKS_BOL_WinksDao::STATUS_ACCEPT )
            {
                $string = $language->text('winks', 'console_wink_wait_item', array('userUrl' => $userUrl, 'displayName' => $displayName));

                if ( $mode == 'mail' || $wink->messageType == 'mail' )
                {
                    $item->setToolbar(array(
                        array(
                            'label' => $language->text('winks', 'send_message'),
                            'url' => OW::getRouter()->urlForRoute('mailbox_conversation', array('convId'=>$wink->getConversationId()))
                        )
                    ));
                }
                else
                {
                    $item->setToolbar(array(
                        array(
                            'label' => $language->text('winks', 'send_message'),
                            'onclick' => 'OW.trigger(\'mailbox.open_dialog\',{convId:' . $wink->getConversationId() . ',opponentId:' . $wink->getUserId() . ',mode:\'' . $mode . '\'});return false;'
                        )
                    ));
                }
            }
            else
            {
                $string = OW::getLanguage()->text('winks', 'console_wink_wait_item', array('userUrl'=> $userUrl, 'displayName'=>$displayName));
                $item->setToolbar(array(
                    array(
                        'label' => $language->text('winks', 'accept_request'),
                        'onclick' => 'Winks.accept(\'' . $item->getKey() . '\',' . $wink->getUserId() . ',' . $wink->getPartnerId() . ');'
                    ),
                    array(
                        'label' => $language->text('winks', 'ignore_request'),
                        'onclick' => 'Winks.ignore(\'' . $item->getKey() . '\',' . $wink->getUserId() . ',' . $wink->getPartnerId() . ');'
                    ),
                    array(
                        'label' => $language->text('winks', 'send_message'),
                        'class' => 'ow_hidden',
                        'id' => 'send-message-' . $item->getKey()
                    )
                ));
            }

            $item->setContent($string);

            if ( $wink->getViewed() === 0 )
            {
                $item->addClass('ow_console_new_message');
            }

            $event->addItem($item->render());
        }

        $this->service->markViewedByIds($viewedIds);
    }
}
